// Copyright (C) 2009,2010,2011,2012 GlavSoft LLC.
// All rights reserved.
//
//-------------------------------------------------------------------------
// This file is part of the TightVNC software.  Please visit our Web site:
//
//                       http://www.tightvnc.com/
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//-------------------------------------------------------------------------
//

#include "QueryConnectionDialog.h"

#include "tvnserver/resource.h"

// Begin Pointdev 2.0.3.0
// New peer information  : NetBIOS computer name, Username, custom message
/*
QueryConnectionDialog::QueryConnectionDialog(const TCHAR *peerAddress,
                                             bool acceptByDefault,
                                             DWORD timeOutInSec)
: m_acceptByDefault(acceptByDefault), m_timeout(timeOutInSec)
{
  m_peerAddress.setString(peerAddress);

  setResourceId(IDD_QUERY_RFB_CONNECTION);
}
*/

QueryConnectionDialog::QueryConnectionDialog(const TCHAR *peerAddress,
											 const TCHAR *peerNetBIOSName,
											 const TCHAR *peerUserName,
											 const TCHAR *peerMessage,
                                             bool acceptByDefault,
                                             DWORD timeOutInSec)
: m_acceptByDefault(acceptByDefault), m_timeout(timeOutInSec)
{
  m_peerAddress.setString(peerAddress);

	// Begin Pointdev 2.0.3.0
	m_peerNetBIOSName.setString(peerNetBIOSName);
	m_peerUserName.setString(peerUserName);
	m_peerMessage.setString(peerMessage);
	// End Pointdev

  setResourceId(IDD_QUERY_RFB_CONNECTION);
}


QueryConnectionDialog::~QueryConnectionDialog()
{
}

BOOL QueryConnectionDialog::onInitDialog()
{
  initControls();

  // Begin Pointdev 2.0.3.0
  m_peerAddressLabel.setText(getPeerInfos().getString());
  m_InfoMessage.setText(getPeerMessage().getString());
  //  m_peerAddressLabel.setText(m_peerAddress.getString());
  // End Pointdev

  if (m_acceptByDefault) {
    m_acceptButton.setFocus();
  } else {
    m_rejectButton.setFocus();
  }

  setDefaultPushButton(m_acceptByDefault ? IDC_ACCEPT_BUTTON : IDC_REJECT_BUTTON);

  updateTimeoutLabel();

  SetTimer(m_ctrlThis.getWindow(), 0, 1000, 0);

  return FALSE;
}

BOOL QueryConnectionDialog::onNotify(UINT controlID, LPARAM data)
{
  return TRUE;
}

BOOL QueryConnectionDialog::onCommand(UINT controlID, UINT notificationID)
{
  switch (controlID) {
  case IDC_ACCEPT_BUTTON:
    onAccept();
    break;
  case IDC_REJECT_BUTTON:
    onReject();
    break;
  }
  return TRUE;
}

BOOL QueryConnectionDialog::onDestroy()
{
  return TRUE;
}

void QueryConnectionDialog::onMessageReceived(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  if (uMsg == WM_TIMER) {
    onTimer();
  }
}

void QueryConnectionDialog::initControls()
{
  HWND window = m_ctrlThis.getWindow();

  m_peerAddressLabel.setWindow(GetDlgItem(window, IDC_IP_EDIT));
  m_acceptButton.setWindow(GetDlgItem(window, IDC_ACCEPT_BUTTON));
  m_rejectButton.setWindow(GetDlgItem(window, IDC_REJECT_BUTTON));
  m_timeoutLabel.setWindow(GetDlgItem(window, IDC_TIMEOUT_LABEL));
  // Begin Pointdev 2.0.3.0
	m_InfoMessage.setWindow(GetDlgItem(window, IDC_MSG_EDIT));	
  // End Pointdev

}

void QueryConnectionDialog::onAccept()
{
  kill(ACCEPT_CHOISE);
}

void QueryConnectionDialog::onReject()
{
  kill(REJECT_CHOISE);
}

void QueryConnectionDialog::onTimer()
{
  if (m_timeout == 0) {
    KillTimer(m_ctrlThis.getWindow(), 0);

    if (m_acceptByDefault) {
      onAccept();
    } else {
      onReject();
    }
  } else {
    m_timeout--;

    updateTimeoutLabel();
  }
}

void QueryConnectionDialog::updateTimeoutLabel()
{
  StringStorage labelText;

  if (m_acceptByDefault) {
    labelText.format(StringTable::getString(IDS_AUTO_ACCEPT_CONNECTION_FORMAT), m_timeout);
  } else {
    labelText.format(StringTable::getString(IDS_AUTO_REJECT_CONNECTION_FORMAT), m_timeout);
  }

  m_timeoutLabel.setText(labelText.getString());
}

// Begin Pointdev 2.0.3.0
StringStorage QueryConnectionDialog::getPeerInfos()
{
	StringStorage peerText;
	
	bool bSep=false;

	if (!m_peerUserName.isEmpty()){
		peerText.appendString(_T("["));
		peerText.appendString(m_peerUserName.getString());
		peerText.appendString(_T("]"));
		bSep=true;
	}

	if (!m_peerNetBIOSName.isEmpty()){
		if (bSep){
			peerText.appendString(_T("\r\n"));
		}
		peerText.appendString(m_peerNetBIOSName.getString());
		bSep=true;
	}

	if (!m_peerAddress.isEmpty()){
		if (bSep){
			peerText.appendString(_T("\r\n"));
		}
		peerText.appendString(m_peerAddress.getString());
		bSep=true;
	}

	return peerText;
}

StringStorage QueryConnectionDialog::getPeerMessage()
{
	// Set the message sent by remote peer
	// Replace "\x1" characters by "\r\n"
	return (m_peerMessage.replaceChars(_T("\x1"), _T("\r\n")).getString());
}
// End Pointdev
