// Begin Pointdev 1.5.3.3 LP
#pragma comment(lib, "Gdiplus.lib")

//#include "vncviewer.h"
#include "ViewerWindow.h"
//#include "Exception.h"
#include "DLGCapture.h"
#include <winuser.h>
#include <GdiPlus.h>
#include "util/DateTime.h"

using namespace Gdiplus;

int ViewerWindow::GetEncoderClsid(const WCHAR* format, CLSID* pClsid)
{
	UINT			num;
	UINT			size;
	ImageCodecInfo	*pImageCodecInfo;

	GetImageEncodersSize(&num, &size);
	if (!size)
		return (-1);
	if (!(pImageCodecInfo = (ImageCodecInfo*)(malloc(size))))
		return (-1);
	GetImageEncoders(num, size, pImageCodecInfo);
	for (UINT j = 0; j < num; ++j)
		if (!wcscmp(pImageCodecInfo[j].MimeType, format))
		{
			*pClsid = pImageCodecInfo[j].Clsid;
			free(pImageCodecInfo);
			return (j);
		}    
	free(pImageCodecInfo);
	return (-1);
}

void ViewerWindow::commandSaveScreen()
{
	TCHAR				fname[MAX_PATH], datestr[MAX_PATH];
	WCHAR				*format;
	DLGCapture			Capture;
	Control				ParentControl;
	CLSID				Clsid;
	ULONG_PTR			gdiplusToken;
	GdiplusStartupInput	gdiplusStartupInput;
	static OPENFILENAME	ofn;
	DateTime datefile;

	//MessageBox(m_hWnd, L"Entre savescreen", L"info", MB_OK);

	datefile=DateTime::now();

	SYSTEMTIME st = {0};
    GetLocalTime(&st);

    StringStorage filterJpegFiles(L"JPEG files (*.jpg)");   
	StringStorage filterBmpFiles(L"BMP files (*.bmp)");
    StringStorage filterPngFiles(L"PNG files (*.png)");
    StringStorage filterAllFiles(L"All files (*.*)");

    TCHAR jpgMask[] = _T("*.jpg");
    TCHAR bmpMask[] = _T("*.bmp");
    TCHAR pngMask[] = _T("*.png");
    TCHAR allMask[] = _T("*.*");

	vector<TCHAR> filter;
    filter.insert(filter.end(), filterJpegFiles.getString(), filterJpegFiles.getString() + filterJpegFiles.getLength() + 1);
    filter.insert(filter.end(), jpgMask, jpgMask + sizeof(jpgMask) / sizeof(TCHAR) - 1);

    filter.push_back(_T(';'));
    filter.push_back(_T('\0'));

    filter.insert(filter.end(), filterBmpFiles.getString(), filterBmpFiles.getString() + filterBmpFiles.getLength() + 1);
	filter.insert(filter.end(), bmpMask, bmpMask + sizeof(bmpMask) / sizeof(TCHAR) - 1);
    filter.push_back(_T(';'));
    filter.push_back(_T('\0'));

    filter.insert(filter.end(), filterPngFiles.getString(), filterPngFiles.getString() + filterPngFiles.getLength() + 1);
	filter.insert(filter.end(), pngMask, pngMask + sizeof(pngMask) / sizeof(TCHAR) - 1);
    filter.push_back(_T(';'));
    filter.push_back(_T('\0'));

	filter.insert(filter.end(), filterAllFiles.getString(), filterAllFiles.getString() + filterAllFiles.getLength() + 1);
    filter.insert(filter.end(), allMask, allMask + sizeof(allMask) / sizeof(TCHAR) - 1);
	filter.push_back(_T('\0'));
    filter.push_back(_T('\0'));

	memset((void*)&ofn, 0, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.lpstrFilter = &filter.front();
	ofn.nMaxFile = MAX_PATH;
	ofn.lpstrDefExt = L"*";

	lstrcpy(fname, m_conConf->m_pdSavePath.getString());

	StringStorage desktopName;
	m_conData->getReducedHost(&desktopName);

	wsprintf(datestr, _T("%.24s-%.4d%.2d%.2d%.2d%.2d%.2d"), desktopName.getString(), st.wYear, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond);

	lstrcat(fname,datestr);

	ofn.hwndOwner = m_hWnd;
	ofn.lpstrFile = fname;
	ofn.Flags = OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT | OFN_PATHMUSTEXIST;
	ParentControl.setWindow(m_hWnd);
	Capture.setParent(&ParentControl);
	Capture.show();
	Sleep(500);
	Capture.kill(0);

	if ( m_conConf->m_pdShowSaveDial && !GetSaveFileName(&ofn))
	{
		DWORD err = CommDlgExtendedError();
		switch(err)
		{
		case 0:	// user canceled
			break;
		case FNERR_INVALIDFILENAME:
			MessageBox(m_hWnd, L"Invalid filename", L"Error saving file",MB_ICONERROR | MB_OK);
			break;
		default:
			m_logWriter.error(L"Error %d from GetSaveFileName\n", err);
			break;
		}
		return;
	}
	else if (!m_conConf->m_pdShowSaveDial)
	{
		lstrcat(ofn.lpstrFile, L".jpg");
		ofn.nFilterIndex = 1;
	}
	GdiplusStartup(&gdiplusToken, &gdiplusStartupInput, NULL);
	if (ofn.nFilterIndex == 2
		|| (ofn.nFilterIndex == 4
			&& !lstrcmp(L"bmp", &ofn.lpstrFile[ofn.nFileExtension])))
		format = L"image/bmp";
	else if (ofn.nFilterIndex == 3
			 || (ofn.nFilterIndex == 4
				 && !lstrcmp(L"png", &ofn.lpstrFile[ofn.nFileExtension])))
		format = L"image/png";
	else
		format = L"image/jpeg";
	if (GetEncoderClsid(format, &Clsid) >= 0)
		CreateScreenShotFile(ofn.lpstrFile, &Clsid);
	else
		m_logWriter.error(L"The %s encoder is not installed.\n", format);

	GdiplusShutdown(gdiplusToken);
}

void ViewerWindow::CreateScreenShotFile(LPWSTR const pszFile, CLSID const *Clsid) const
{
	size_t				len;
	ULONG				quality;
	EncoderParameters	encoderParameters;

	len = lstrlen(pszFile) + 1;
	quality = 95;
	encoderParameters.Count = 1;
	encoderParameters.Parameter[0].Guid = EncoderQuality;
	encoderParameters.Parameter[0].Type = EncoderParameterValueTypeLong;
	encoderParameters.Parameter[0].NumberOfValues = 1;
	encoderParameters.Parameter[0].Value = &quality;

	if (m_dsktWnd.m_cBitmap)
	{
		HPALETTE m_hPalette=NULL;

		Bitmap				*BmpImage=NULL;
		BmpImage = new Bitmap(m_dsktWnd.m_cBitmap, m_hPalette);
		BmpImage->Save(pszFile, Clsid, &encoderParameters);
	
		delete BmpImage;	
	}
}
// End Pointdev
