// Copyright (C) 2011,2012 GlavSoft LLC.
// All rights reserved.
//
//-------------------------------------------------------------------------
// This file is part of the TightVNC software.  Please visit our Web site:
//
//                       http://www.tightvnc.com/
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//-------------------------------------------------------------------------
//

#include "client-config-lib/ViewerConfig.h"
#include "log-writer/LogWriter.h"
#include "TvnViewer.h"
#include "ConnectionData.h"
#include "ConnectionListener.h"
#include "ViewerCmdLine.h"
#include "util/ResourceLoader.h"

int APIENTRY _tWinMain(HINSTANCE hInstance, HINSTANCE,
                       LPTSTR lpCmdLine, int nCmdShow)
{
  ViewerSettingsManager::initInstance(RegistryPaths::VIEWER_PATH);
  SettingsManager *sm = ViewerSettingsManager::getInstance();

  ViewerConfig config(RegistryPaths::VIEWER_PATH);
  config.loadFromStorage(sm);

  try {
    config.initLog(LogNames::LOG_DIR_NAME, LogNames::VIEWER_LOG_FILE_STUB_NAME);
  } catch (...) {
  }

  LogWriter logWriter(config.getLogger());

  // resource-loader initialization
  ResourceLoader resourceLoader(hInstance);

  logWriter.debug(_T("main()"));
  logWriter.debug(_T("loading settings from storage completed"));
  logWriter.debug(_T("Log initialization completed"));

  ConnectionConfig conConf;
  ConnectionData condata;
  bool isListening = false;
  ViewerCmdLine cmd(&condata, &conConf, &config, &isListening);

  int result = 0;
  try {
    cmd.parse();

	//Begin Pointdev
	//if isListening then we change WINDOW_CLASS_NAME by added " deamon"
	//to recognize window class name in IDEAL Administration (Internet Remote Control)
	TCHAR winname[MAX_PATH];
	if (isListening)
		wsprintf(winname,_T("%s Daemon"),ApplicationNames::WINDOW_CLASS_NAME);
	else
		wsprintf(winname,_T("%s"),ApplicationNames::WINDOW_CLASS_NAME);
	//End Pointdev

    TvnViewer tvnViewer(hInstance,
                        ApplicationNames::WINDOW_CLASS_NAME,
                        WindowNames::TVN_WINDOW_CLASS_NAME);

    if (isListening) {
      // FIXME: set listening connection options.
      //pointdev : LISTEN pb, impossible de paramtrer le port par dfaut
	  //Begin Pointdev
		int mlistenport=cmd.getListenPort();
		if (mlistenport==0)
			mlistenport=ConnectionListener::DEFAULT_PORT;
		tvnViewer.startListening(mlistenport);
        //tvnViewer.startListening(ConnectionListener::DEFAULT_PORT);
	  //End Pointdev
    } else if (!condata.isEmpty()) {
      tvnViewer.newConnection(&condata, &conConf);
    } else {
      tvnViewer.showLoginDialog();
    }
    result = tvnViewer.run();
  } catch (const CommandLineFormatException &exception) {
    StringStorage strError(exception.getMessage());
    MessageBox(0,
               strError.getString(),
               ProductNames::VIEWER_PRODUCT_NAME,
               MB_OK | MB_ICONERROR);
    return 0;
  } catch (const CommandLineFormatHelp &) {
    cmd.onHelp();
    return 0;
  } catch (const Exception &ex) {
    MessageBox(0,
               StringTable::getString(IDS_UNKNOWN_ERROR_IN_VIEWER),
               ProductNames::VIEWER_PRODUCT_NAME,
               MB_OK | MB_ICONERROR);
    logWriter.debug(ex.getMessage());
  }

  return result;
}
